@@ -55,7 +55,7 @@ void gbRenderLine()
         bank1 = &gbVram[0x2000];
     } else {
         bank0 = &gbMemory[0x8000];
-        bank1 = nullptr;
+        bank1 = NULL;
     }
 
     int tile_map = 0x1800;
@@ -92,7 +92,7 @@ void gbRenderLine()
     int tile_map_address = tile_map_line_y + tx;
 
     uint8_t _attrs = 0;
-    if (bank1 != nullptr)
+    if (bank1 != NULL)
         _attrs = bank1[tile_map_address];
 
     uint8_t tile = bank0[tile_map_address];
@@ -406,7 +406,7 @@ void gbDrawSpriteTile(int tile, int x, int y, int t, int flags,
         bank1 = &gbVram[0x2000];
     } else {
         bank0 = &gbMemory[0x8000];
-        bank1 = nullptr;
+        bank1 = NULL;
     }
 
     int SpritesTicks = gbSpritesTicks[x + 8] * (gbSpeed ? 2 : 4);
@@ -434,7 +434,9 @@ void gbDrawSpriteTile(int tile, int x, int y, int t, int flags,
     int a = 0;
     int b = 0;
 
-    if (gbCgbMode && (flags & 0x08)) {
+    // In CGB mode with DMG compatibility (0xff6c & 1), ignore the VRAM bank flag
+    // because DMG games don't use VRAM banks and the flag bit may be set for other purposes
+    if (gbCgbMode && (flags & 0x08) && !(gbMemory[0xff6c] & 1)) {
         a = bank1[address++];
         b = bank1[address++];
     } else {
