@@ -10,6 +10,7 @@
 #include "scrc32.h"
 
 #include "components/filters_agb/filters_agb.h"
+#include "components/filters_cgb/filters_cgb.h"
 #include "components/filters_interframe/interframe.h"
 #include "core/base/check.h"
 #include "core/base/system.h"
@@ -184,9 +185,9 @@ static void* gb_rtcdata_prt(void)
     case gbCartData::MapperType::kGameShark:
     case gbCartData::MapperType::kUnknown:
         VBAM_NOTREACHED();
-        return nullptr;
+        return NULL;
     }
-    return nullptr;
+    return NULL;
 }
 
 static size_t gb_rtcdata_size(void)
@@ -611,7 +612,15 @@ void retro_init(void)
     if (environ_cb(RETRO_ENVIRONMENT_GET_SYSTEM_DIRECTORY, &dir) && dir)
         snprintf(retro_system_directory, sizeof(retro_system_directory), "%s", dir);
 
-#ifdef FRONTEND_SUPPORTS_RGB565
+#ifdef FRONTEND_SUPPORT_BGR1555
+    systemColorDepth = 16;
+    systemRedShift = 0;
+    systemGreenShift = 5;
+    systemBlueShift = 10;
+    enum retro_pixel_format rgb1555 = RETRO_PIXEL_FORMAT_0RGB1555;
+    if (environ_cb(RETRO_ENVIRONMENT_SET_PIXEL_FORMAT, &rgb1555) && log_cb)
+        log_cb(RETRO_LOG_INFO, "Frontend supports BGR1555 - will use that instead of XRGB1555.\n");
+#elif defined(FRONTEND_SUPPORTS_RGB565)
     systemColorDepth = 16;
     systemRedShift = 11;
     systemGreenShift = 6;
@@ -970,14 +979,16 @@ static void gb_init(void)
 void retro_deinit(void)
 {
     emulating = 0;
-    core->emuCleanUp();
+    if (core)
+        core->emuCleanUp();
     soundShutdown();
     libretro_supports_bitmasks = false;
 }
 
 void retro_reset(void)
 {
-    core->emuReset();
+    if (core)
+        core->emuReset();
     set_gbPalette();
 }
 
@@ -1012,6 +1023,11 @@ static int option_analogDeadzone;
 static int option_gyroSensitivity, option_tiltSensitivity;
 static bool option_swapAnalogSticks;
 
+static int color_mode = 0;
+static int prev_color_mode = 0;
+static float color_change = 0.0f;
+static float prev_color_change = 0.0f;
+
 static void update_variables(bool startup)
 {
     struct retro_variable var = { NULL, NULL };
@@ -1101,13 +1117,9 @@ static void update_variables(bool startup)
     if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value) {
         if (strcmp(var.value, "auto") == 0) {
             gbBorderAutomatic = true;
-        }
-        else if (!strcmp(var.value, "enabled")) {
-            gbBorderAutomatic = false;
-            gbBorderOn = false;
-        } else { // disabled
-            gbBorderOn = true;
+        } else {
             gbBorderAutomatic = false;
+            gbBorderOn = (bool)(!strcmp(var.value, "enabled"));
         }
 
         if ((type == IMAGE_GB) && !startup)
@@ -1215,14 +1227,47 @@ static void update_variables(bool startup)
         gbColorOption = (!strcmp(var.value, "enabled"));
     }
 
+    var.key = "vbam_lcdfilter_type";
+    var.value = NULL;
+
+    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value) {
+        prev_color_mode = color_mode;
+        if (!strcmp(var.value, "sRGB"))
+            color_mode = 0;
+        else if (!strcmp(var.value, "DCI"))
+            color_mode = 1;
+        else if (!strcmp(var.value, "Rec2020"))
+            color_mode = 2;
+    }
+
+    var.key = "vbam_color_change";
+    var.value = NULL;
+
+    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value) {
+        prev_color_change = color_change;
+        color_change = ((float)atoi(var.value)) / 100;
+    }
+
     var.key = "vbam_lcdfilter";
     var.value = NULL;
 
     if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value) {
         bool prev_lcdfilter = option_lcdfilter;
         option_lcdfilter = (!strcmp(var.value, "enabled")) ? true : false;
-        if (prev_lcdfilter != option_lcdfilter)
-            gbafilter_update_colors(option_lcdfilter);
+        if ((prev_color_change != color_change) || (prev_color_mode != color_mode)) {
+            if (type == IMAGE_GBA) {
+                gbafilter_set_params(color_mode, color_change);
+            } else {
+                gbcfilter_set_params(color_mode, color_change);
+            }
+        }
+        if ((prev_lcdfilter != option_lcdfilter) || (prev_color_change != color_change) || (prev_color_mode != color_mode)) {
+            if (type == IMAGE_GBA) {
+                gbafilter_update_colors(option_lcdfilter);
+            } else {
+                gbcfilter_update_colors(option_lcdfilter);
+            }
+        }
     }
 
     var.key = "vbam_interframeblending";
@@ -1492,6 +1537,8 @@ size_t retro_serialize_size(void)
 
 bool retro_serialize(void* data, size_t size)
 {
+    if (!core)
+        return false;
     if (size == serialize_size)
         return core->emuWriteState((uint8_t*)data);
     return false;
@@ -1499,6 +1546,8 @@ bool retro_serialize(void* data, size_t size)
 
 bool retro_unserialize(const void* data, size_t size)
 {
+    if (!core)
+        return false;
     if (size == serialize_size)
         return core->emuReadState((uint8_t*)data);
     return false;
@@ -1787,8 +1836,10 @@ bool systemCanChangeSoundQuality(void)
 void systemDrawScreen(void)
 {
     unsigned pitch = systemWidth * (systemColorDepth >> 3);
+    
     if (ifb_filter_func)
         ifb_filter_func(g_pix, pitch, systemWidth, systemHeight);
+
     video_cb(g_pix, systemWidth, systemHeight, pitch);
 }
 
