@@ -18,17 +18,21 @@
 
 // Parts adapted from VBA-H (VBA for Hackers) by LabMaster
 
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cctype>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
-#include "../common/Port.h"
-#include "../gba/GBA.h"
-#include "../gba/Sound.h"
-#include "../gba/armdis.h"
-#include "../gba/elf.h"
-#include "exprNode.h"
+#include "core/base/port.h"
+#include "core/gba/gbaCpu.h"
+#include "core/gba/gbaCpuArmDis.h"
+#include "core/gba/gbaElf.h"
+#include "core/gba/gbaSound.h"
+#include "sdl/exprNode.h"
+
+#if __STDC_WANT_SECURE_LIB__
+#define snprintf sprintf_s
+#endif
 
 extern bool debugger;
 extern int emulating;
@@ -289,11 +293,11 @@ static void debuggerPrintBaseType(Type* t, uint32_t value, uint32_t location,
     }
 
     if (t->size == 8) {
-        u64 value = 0;
+        uint64_t value = 0;
         if (type == LOCATION_memory) {
-            value = debuggerReadMemory(location) | ((u64)debuggerReadMemory(location + 4) << 32);
+            value = debuggerReadMemory(location) | ((uint64_t)debuggerReadMemory(location + 4) << 32);
         } else if (type == LOCATION_register) {
-            value = reg[location].I | ((u64)reg[location + 1].I << 32);
+            value = reg[location].I | ((uint64_t)reg[location + 1].I << 32);
         }
         switch (t->encoding) {
         case DW_ATE_signed:
@@ -377,11 +381,11 @@ static const char* debuggerPrintType(Type* t)
             strcpy(buffer, debuggerPrintType(t->pointer));
         else
             strcpy(buffer, "void");
-        sprintf(buffer2, "%s *", buffer);
+        snprintf(buffer2, sizeof(buffer2), "%s *", buffer);
         return buffer2;
     } else if (t->type == TYPE_reference) {
         strcpy(buffer, debuggerPrintType(t->pointer));
-        sprintf(buffer2, "%s &", buffer);
+        snprintf(buffer2, sizeof(buffer2), "%s &", buffer);
         return buffer2;
     }
     return t->name;
@@ -1364,7 +1368,7 @@ static void debuggerDisassembleArm(FILE* f, uint32_t pc, int count)
         if (l > len)
             len = l;
     }
-    sprintf(format, "%%08x %%-%ds %%s\n", len);
+    snprintf(format, sizeof(format), "%%08x %%-%ds %%s\n", len);
     for (i = 0; i < count; i++) {
         uint32_t addr = pc;
         pc += disArm(pc, buffer, 4096, 2);
@@ -1383,7 +1387,7 @@ static void debuggerDisassembleThumb(FILE* f, uint32_t pc, int count)
         if (l > len)
             len = l;
     }
-    sprintf(format, "%%08x %%-%ds %%s\n", len);
+    snprintf(format, sizeof(format), "%%08x %%-%ds %%s\n", len);
 
     for (i = 0; i < count; i++) {
         uint32_t addr = pc;
@@ -1590,8 +1594,8 @@ void debuggerDoSearch()
                 SearchStart = 0x02000000;
                 continue;
             } else {
-                start = bios + (SearchStart & 0x3FFF);
-                end = bios + 0x3FFF;
+                start = g_bios + (SearchStart & 0x3FFF);
+                end = g_bios + 0x3FFF;
                 break;
             };
         case 2:
@@ -1599,8 +1603,8 @@ void debuggerDoSearch()
                 SearchStart = 0x03000000;
                 continue;
             } else {
-                start = workRAM + (SearchStart & 0x3FFFF);
-                end = workRAM + 0x3FFFF;
+                start = g_workRAM + (SearchStart & 0x3FFFF);
+                end = g_workRAM + 0x3FFFF;
                 break;
             };
         case 3:
@@ -1608,8 +1612,8 @@ void debuggerDoSearch()
                 SearchStart = 0x04000000;
                 continue;
             } else {
-                start = internalRAM + (SearchStart & 0x7FFF);
-                end = internalRAM + 0x7FFF;
+                start = g_internalRAM + (SearchStart & 0x7FFF);
+                end = g_internalRAM + 0x7FFF;
                 break;
             };
         case 4:
@@ -1617,8 +1621,8 @@ void debuggerDoSearch()
                 SearchStart = 0x05000000;
                 continue;
             } else {
-                start = ioMem + (SearchStart & 0x3FF);
-                end = ioMem + 0x3FF;
+                start = g_ioMem + (SearchStart & 0x3FF);
+                end = g_ioMem + 0x3FF;
                 break;
             };
         case 5:
@@ -1626,8 +1630,8 @@ void debuggerDoSearch()
                 SearchStart = 0x06000000;
                 continue;
             } else {
-                start = paletteRAM + (SearchStart & 0x3FF);
-                end = paletteRAM + 0x3FF;
+                start = g_paletteRAM + (SearchStart & 0x3FF);
+                end = g_paletteRAM + 0x3FF;
                 break;
             };
         case 6:
@@ -1635,8 +1639,8 @@ void debuggerDoSearch()
                 SearchStart = 0x07000000;
                 continue;
             } else {
-                start = vram + (SearchStart & 0x1FFFF);
-                end = vram + 0x1FFFF;
+                start = g_vram + (SearchStart & 0x1FFFF);
+                end = g_vram + 0x1FFFF;
                 break;
             };
         case 7:
@@ -1644,8 +1648,8 @@ void debuggerDoSearch()
                 SearchStart = 0x08000000;
                 continue;
             } else {
-                start = oam + (SearchStart & 0x3FF);
-                end = oam + 0x3FF;
+                start = g_oam + (SearchStart & 0x3FF);
+                end = g_oam + 0x3FF;
                 break;
             };
         case 8:
@@ -1655,8 +1659,8 @@ void debuggerDoSearch()
         case 12:
         case 13:
             if (final <= 0x09FFFFFF) {
-                start = rom + (SearchStart & 0x01FFFFFF);
-                end = rom + 0x01FFFFFF;
+                start = g_rom + (SearchStart & 0x01FFFFFF);
+                end = g_rom + 0x01FFFFFF;
                 break;
             };
         default:
@@ -1698,22 +1702,22 @@ void debuggerDoSearch()
 
 unsigned int AddressToGBA(uint8_t* mem)
 {
-    if (mem >= &bios[0] && mem <= &bios[0x3fff])
-        return 0x00000000 + (mem - &bios[0]);
-    else if (mem >= &workRAM[0] && mem <= &workRAM[0x3ffff])
-        return 0x02000000 + (mem - &workRAM[0]);
-    else if (mem >= &internalRAM[0] && mem <= &internalRAM[0x7fff])
-        return 0x03000000 + (mem - &internalRAM[0]);
-    else if (mem >= &ioMem[0] && mem <= &ioMem[0x3ff])
-        return 0x04000000 + (mem - &ioMem[0]);
-    else if (mem >= &paletteRAM[0] && mem <= &paletteRAM[0x3ff])
-        return 0x05000000 + (mem - &paletteRAM[0]);
-    else if (mem >= &vram[0] && mem <= &vram[0x1ffff])
-        return 0x06000000 + (mem - &vram[0]);
-    else if (mem >= &oam[0] && mem <= &oam[0x3ff])
-        return 0x07000000 + (mem - &oam[0]);
-    else if (mem >= &rom[0] && mem <= &rom[0x1ffffff])
-        return 0x08000000 + (mem - &rom[0]);
+    if (mem >= &g_bios[0] && mem <= &g_bios[0x3fff])
+        return 0x00000000 + (mem - &g_bios[0]);
+    else if (mem >= &g_workRAM[0] && mem <= &g_workRAM[0x3ffff])
+        return 0x02000000 + (mem - &g_workRAM[0]);
+    else if (mem >= &g_internalRAM[0] && mem <= &g_internalRAM[0x7fff])
+        return 0x03000000 + (mem - &g_internalRAM[0]);
+    else if (mem >= &g_ioMem[0] && mem <= &g_ioMem[0x3ff])
+        return 0x04000000 + (mem - &g_ioMem[0]);
+    else if (mem >= &g_paletteRAM[0] && mem <= &g_paletteRAM[0x3ff])
+        return 0x05000000 + (mem - &g_paletteRAM[0]);
+    else if (mem >= &g_vram[0] && mem <= &g_vram[0x1ffff])
+        return 0x06000000 + (mem - &g_vram[0]);
+    else if (mem >= &g_oam[0] && mem <= &g_oam[0x3ff])
+        return 0x07000000 + (mem - &g_oam[0]);
+    else if (mem >= &g_rom[0] && mem <= &g_rom[0x1ffffff])
+        return 0x08000000 + (mem - &g_rom[0]);
     else
         return 0xFFFFFFFF;
 };
@@ -1747,7 +1751,7 @@ static void debuggerRegisters(int, char**)
     char* command[3] = { m, 0, one };
     char buffer[10];
 
-#ifdef BKPT_SUPPORT
+#ifdef VBAM_ENABLE_DEBUGGER
     if (debugger_last) {
         printf("R00=%08x R04=%08x R08=%08x R12=%08x\n",
             oldreg[0], oldreg[4], oldreg[8], oldreg[12]);
@@ -1781,7 +1785,7 @@ static void debuggerRegisters(int, char**)
         ((!(reg[16].I & 0x40)) ? '.' : 'F'),
         (armState ? '.' : 'T'),
         armMode);
-    sprintf(buffer, "%08x", armState ? reg[15].I - 4 : reg[15].I - 2);
+    snprintf(buffer, sizeof(buffer), "%08x", armState ? reg[15].I - 4 : reg[15].I - 2);
     command[1] = buffer;
     debuggerDisassemble(3, command);
 }
