@@ -83,6 +83,15 @@ static const std::array<wxString, kNbRenderMethods> kRenderMethodStrings = {
 #endif
 };
 
+// These MUST follow the same order as the definitions of the enum.
+// Adding an option without adding to this array will result in a compiler
+// error since kNbColorCorrectionProfiles is automatically updated.
+static const std::array<wxString, kNbColorCorrectionProfiles> kColorCorrectionProfileStrings = {
+    "srgb",
+    "dci",
+    "rec2020"
+};
+
 // These MUST follow the same order as the definitions of the enum above.
 // Adding an option without adding to this array will result in a compiler
 // error since kNbAudioApis is automatically updated.
@@ -141,7 +150,8 @@ wxString AllEnumValuesForArray(const std::array<wxString, SIZE>& input) {
 std::array<Option, kNbOptions>& Option::All() {
     struct OwnedOptions {
         /// Display
-        bool bilinear = true;
+        bool bilinear = false;
+        bool sdl_pixel_art = false;
         Filter filter = Filter::kNone;
         wxString filter_plugin = wxEmptyString;
         Interframe interframe = Interframe::kNone;
@@ -158,27 +168,32 @@ std::array<Option, kNbOptions>& Option::All() {
         RenderMethod render_method = RenderMethod::kOpenGL;
 #endif
 #endif
+
+        ColorCorrectionProfile color_correction_profile = ColorCorrectionProfile::kSRGB;
+
         double video_scale = 3;
         bool retain_aspect = true;
 
         /// GB
         wxString gb_bios = wxEmptyString;
         bool colorizer_hack = false;
-        bool gb_lcd_filter = false;
+        bool gb_lcd_filter = true;
         wxString gbc_bios = wxEmptyString;
         bool print_auto_page = true;
         bool print_screen_cap = false;
         wxString gb_rom_dir = wxEmptyString;
         wxString gbc_rom_dir = wxEmptyString;
+        uint32_t gb_lighten = 0;
 
         /// GBA
-        bool gba_lcd_filter = false;
+        bool gba_lcd_filter = true;
 #ifndef NO_LINK
         bool link_auto = false;
         bool link_hacks = true;
         bool link_proto = false;
 #endif
         wxString gba_rom_dir;
+        uint32_t gba_darken = 37;
 
         /// Core
         bool agb_print = false;
@@ -205,7 +220,7 @@ std::array<Option, kNbOptions>& Option::All() {
         wxString recording_dir = wxEmptyString;
         wxString screenshot_dir = wxEmptyString;
         wxString state_dir = wxEmptyString;
-        bool statusbar = false;
+        bool statusbar = true;
         uint32_t ini_version = kIniLatestVersion;
 
         /// Joypad
@@ -266,6 +281,7 @@ std::array<Option, kNbOptions>& Option::All() {
     static std::array<Option, kNbOptions> g_all_opts = {
         /// Display
         Option(OptionID::kDispBilinear, &g_owned_opts.bilinear),
+        Option(OptionID::kDispSDLPixelArt, &g_owned_opts.sdl_pixel_art),
         Option(OptionID::kDispFilter, &g_owned_opts.filter),
         Option(OptionID::kDispFilterPlugin, &g_owned_opts.filter_plugin),
         Option(OptionID::kDispIFB, &g_owned_opts.interframe),
@@ -276,6 +292,7 @@ std::array<Option, kNbOptions>& Option::All() {
         Option(OptionID::kDispScale, &g_owned_opts.video_scale, 1, 6),
         Option(OptionID::kDispStretch, &g_owned_opts.retain_aspect),
         Option(OptionID::kSDLRenderer, &g_owned_opts.sdlrenderer),
+        Option(OptionID::kDispColorCorrectionProfile, &g_owned_opts.color_correction_profile),
 
         /// GB
         Option(OptionID::kGBBiosFile, &g_owned_opts.gb_bios),
@@ -290,6 +307,7 @@ std::array<Option, kNbOptions>& Option::All() {
         Option(OptionID::kGBPrintScreenCap, &g_owned_opts.print_screen_cap),
         Option(OptionID::kGBROMDir, &g_owned_opts.gb_rom_dir),
         Option(OptionID::kGBGBCROMDir, &g_owned_opts.gbc_rom_dir),
+        Option(OptionID::kGBLighten, &g_owned_opts.gb_lighten, 0, 100),
 
         /// GBA
         Option(OptionID::kGBABiosFile, &gopts.gba_bios),
@@ -305,6 +323,7 @@ std::array<Option, kNbOptions>& Option::All() {
         Option(OptionID::kGBALinkType, &gopts.gba_link_type, 0, 5),
 #endif
         Option(OptionID::kGBAROMDir, &g_owned_opts.gba_rom_dir),
+        Option(OptionID::kGBADarken, &g_owned_opts.gba_darken, 0, 100),
 
         /// General
         Option(OptionID::kGenAutoLoadLastState, &g_owned_opts.autoload_state),
@@ -410,6 +429,7 @@ namespace internal {
 const std::array<OptionData, kNbOptions + 1> kAllOptionsData = {
     /// Display
     OptionData{"Display/Bilinear", "Bilinear", _("Use bilinear filter with 3d renderer")},
+    OptionData{"Display/SDLPixelArt", "SDLPixelArt", _("Use the SDL pixel art filter with an SDL renderer")},
     OptionData{"Display/Filter", "", _("Full-screen filter to apply")},
     OptionData{"Display/FilterPlugin", "", _("Filter plugin library")},
     OptionData{"Display/IFB", "", _("Interframe blending function")},
@@ -422,6 +442,7 @@ const std::array<OptionData, kNbOptions + 1> kAllOptionsData = {
     OptionData{"Display/Scale", "", _("Default scale factor")},
     OptionData{"Display/Stretch", "RetainAspect", _("Retain aspect ratio when resizing")},
     OptionData{"Display/SDLRenderer", "", _("SDL renderer")},
+    OptionData{"Display/ColorCorrectionProfile", "", _("Color correction profile")},
 
     /// GB
     OptionData{"GB/BiosFile", "", _("BIOS file to use for Game Boy, if enabled")},
@@ -445,6 +466,7 @@ const std::array<OptionData, kNbOptions + 1> kAllOptionsData = {
                  "suffix")},
     OptionData{"GB/ROMDir", "", _("Directory to look for ROM files")},
     OptionData{"GB/GBCROMDir", "", _("Directory to look for Game Boy Color ROM files")},
+    OptionData{"GB/GBCLighten", "", _("Color lightness factor")},
 
     /// GBA
     OptionData{"GBA/BiosFile", "", _("BIOS file to use, if enabled")},
@@ -472,6 +494,7 @@ const std::array<OptionData, kNbOptions + 1> kAllOptionsData = {
     OptionData{"GBA/LinkType", "LinkType", _("Link cable type")},
 #endif
     OptionData{"GBA/ROMDir", "", _("Directory to look for ROM files")},
+    OptionData{"GBA/GBADarken", "", _("Color darkness factor")},
 
     /// General
     OptionData{"General/AutoLoadLastState", "", _("Automatically load last saved state")},
@@ -652,6 +675,12 @@ wxString RenderMethodToString(const RenderMethod& value) {
     return kRenderMethodStrings[size_value];
 }
 
+wxString ColorCorrectionProfileToString(const ColorCorrectionProfile& value) {
+    const size_t size_value = static_cast<size_t>(value);
+    VBAM_CHECK(size_value < kNbColorCorrectionProfiles);
+    return kColorCorrectionProfileStrings[size_value];
+}
+
 wxString AudioApiToString(const AudioApi& value) {
     const size_t size_value = static_cast<size_t>(value);
     VBAM_CHECK(size_value < kNbAudioApis);
@@ -725,6 +754,27 @@ RenderMethod StringToRenderMethod(const wxString& config_name,
     return iter->second;
 }
 
+ColorCorrectionProfile StringToColorCorrectionProfile(const wxString& config_name,
+                                  const wxString& input) {
+    static const std::map<wxString, ColorCorrectionProfile> kStringToColorCorrectionProfile([] {
+        std::map<wxString, ColorCorrectionProfile> string_to_color_correction_profile;
+        for (size_t i = 0; i < kNbColorCorrectionProfiles; i++) {
+            string_to_color_correction_profile.emplace(kColorCorrectionProfileStrings[i], static_cast<ColorCorrectionProfile>(i));
+        }
+        VBAM_CHECK(string_to_color_correction_profile.size() == kNbColorCorrectionProfiles);
+        return string_to_color_correction_profile;
+    }());
+
+    const auto iter = kStringToColorCorrectionProfile.find(input);
+    if (iter == kStringToColorCorrectionProfile.end()) {
+        wxLogWarning(_("Invalid value %s for option %s; valid values are %s"),
+                     input, config_name,
+                     AllEnumValuesForType(Option::Type::kColorCorrectionProfile));
+        return ColorCorrectionProfile::kSRGB;
+    }
+    return iter->second;
+}
+
 AudioApi StringToAudioApi(const wxString& config_name, const wxString& input) {
     static const std::map<wxString, AudioApi> kStringToAudioApi([] {
         std::map<wxString, AudioApi> string_to_audio_api;
@@ -778,6 +828,10 @@ wxString AllEnumValuesForType(Option::Type type) {
             static const wxString kAllRenderValues(AllEnumValuesForArray(kRenderMethodStrings));
             return kAllRenderValues;
         }
+        case Option::Type::kColorCorrectionProfile: {
+            static const wxString kAllColorCorrectionValues(AllEnumValuesForArray(kColorCorrectionProfileStrings));
+            return kAllColorCorrectionValues;
+        }
         case Option::Type::kAudioApi: {
             static const wxString kAllAudioApiValues(AllEnumValuesForArray(kAudioApiStrings));
             return kAllAudioApiValues;
@@ -811,6 +865,8 @@ size_t MaxForType(Option::Type type) {
             return kNbInterframes;
         case Option::Type::kRenderMethod:
             return kNbRenderMethods;
+        case Option::Type::kColorCorrectionProfile:
+            return kNbColorCorrectionProfiles;
         case Option::Type::kAudioApi:
             return kNbAudioApis;
         case Option::Type::kAudioRate:
