@@ -8,6 +8,7 @@
 #include <wx/log.h>
 #include <wx/object.h>
 #include <wx/radiobut.h>
+#include <wx/slider.h>
 #include <wx/textctrl.h>
 #include <wx/valnum.h>
 
@@ -18,6 +19,7 @@
 #include "wx/config/option.h"
 #include "wx/dialogs/base-dialog.h"
 #include "wx/rpi.h"
+#include "wx/wayland.h"
 #include "wx/widgets/option-validator.h"
 #include "wx/widgets/render-plugin.h"
 #include "wx/wxvbam.h"
@@ -223,6 +225,41 @@ private:
     const config::RenderMethod render_method_;
 };
 
+// Validator for a wxRadioButton with a ColorCorrectionProfile value.
+class ColorCorrectionProfileValidator : public widgets::OptionValidator {
+public:
+    explicit ColorCorrectionProfileValidator(config::ColorCorrectionProfile color_correction_profile)
+        : OptionValidator(config::OptionID::kDispColorCorrectionProfile),
+          color_correction_profile_(color_correction_profile) {
+        VBAM_CHECK(color_correction_profile != config::ColorCorrectionProfile::kLast);
+    }
+    ~ColorCorrectionProfileValidator() override = default;
+
+private:
+    // OptionValidator implementation.
+    wxObject* Clone() const override {
+        return new ColorCorrectionProfileValidator(color_correction_profile_);
+    }
+
+    bool IsWindowValueValid() override { return true; }
+
+    bool WriteToWindow() override {
+        wxDynamicCast(GetWindow(), wxRadioButton)
+            ->SetValue(option()->GetColorCorrectionProfile() == color_correction_profile_);
+        return true;
+    }
+
+    bool WriteToOption() override {
+        if (wxDynamicCast(GetWindow(), wxRadioButton)->GetValue()) {
+            return option()->SetColorCorrectionProfile(color_correction_profile_);
+        }
+
+        return true;
+    }
+
+    const config::ColorCorrectionProfile color_correction_profile_;
+};
+
 class PluginSelectorValidator : public widgets::OptionValidator {
 public:
     PluginSelectorValidator()
@@ -266,6 +303,22 @@ private:
 
 }  // namespace
 
+// Helper function to update slider tooltip with its current value
+static void UpdateSliderTooltip(wxSlider* slider, wxCommandEvent& event) {
+    if (slider) {
+        slider->SetToolTip(wxString::Format("%d", slider->GetValue()));
+    }
+    event.Skip();
+}
+
+// Helper function to update slider tooltip on mouse enter
+static void UpdateSliderTooltipOnHover(wxSlider* slider, wxMouseEvent& event) {
+    if (slider) {
+        slider->SetToolTip(wxString::Format("%d", slider->GetValue()));
+    }
+    event.Skip();
+}
+
 // static
 DisplayConfig* DisplayConfig::NewInstance(wxWindow* parent) {
     VBAM_CHECK(parent);
@@ -348,9 +401,86 @@ DisplayConfig::DisplayConfig(wxWindow* parent)
     GetValidatedChild("OutputDirect3D")->Hide();
 #endif
 
+    sdlrenderer_label_ = GetValidatedChild<wxControl>("SDLRendererLab");
     sdlrenderer_selector_ = GetValidatedChild<wxChoice>("SDLRenderer");
     sdlrenderer_selector_->SetValidator(SDLDevicesValidator());
 
+    sdlpixelart_checkbox_ = GetValidatedChild<wxControl>("SDLPixelArt");
+#if !defined(ENABLE_SDL3) || !defined(HAVE_SDL3_PIXELART)
+    sdlpixelart_checkbox_->Hide();
+#else
+    wxDynamicCast(sdlpixelart_checkbox_, wxCheckBox)
+        ->SetValidator(
+            widgets::OptionBoolValidator(config::OptionID::kDispSDLPixelArt));
+#endif
+
+    // Bind event handlers to all output module radio buttons
+    GetValidatedChild("OutputSimple")->Bind(wxEVT_RADIOBUTTON,
+        &DisplayConfig::UpdateSDLOptionsVisibility, this);
+    GetValidatedChild("OutputSDL")->Bind(wxEVT_RADIOBUTTON,
+        &DisplayConfig::UpdateSDLOptionsVisibility, this);
+#ifdef __WXMAC__
+    GetValidatedChild("OutputQuartz2D")->Bind(wxEVT_RADIOBUTTON,
+        &DisplayConfig::UpdateSDLOptionsVisibility, this);
+#ifndef NO_METAL
+    GetValidatedChild("OutputMetal")->Bind(wxEVT_RADIOBUTTON,
+        &DisplayConfig::UpdateSDLOptionsVisibility, this);
+#endif
+#endif
+#ifndef NO_OGL
+#if defined(HAVE_WAYLAND_SUPPORT) && !defined(HAVE_WAYLAND_EGL)
+    if (!IsWayland()) {
+        GetValidatedChild("OutputOpenGL")->Bind(wxEVT_RADIOBUTTON,
+            &DisplayConfig::UpdateSDLOptionsVisibility, this);
+    }
+#else
+    GetValidatedChild("OutputOpenGL")->Bind(wxEVT_RADIOBUTTON,
+        &DisplayConfig::UpdateSDLOptionsVisibility, this);
+#endif
+#endif
+#if defined(__WXMSW__) && !defined(NO_D3D)
+    GetValidatedChild("OutputDirect3D")->Bind(wxEVT_RADIOBUTTON,
+        &DisplayConfig::UpdateSDLOptionsVisibility, this);
+#endif
+
+    wxWindow* color_profile_srgb = GetValidatedChild("ColorProfileSRGB");
+    color_profile_srgb->SetValidator(
+        ColorCorrectionProfileValidator(config::ColorCorrectionProfile::kSRGB));
+
+    wxWindow* color_profile_dci = GetValidatedChild("ColorProfileDCI");
+    color_profile_dci->SetValidator(
+        ColorCorrectionProfileValidator(config::ColorCorrectionProfile::kDCI));
+
+    wxWindow* color_profile_rec2020 = GetValidatedChild("ColorProfileRec2020");
+    color_profile_rec2020->SetValidator(
+        ColorCorrectionProfileValidator(config::ColorCorrectionProfile::kRec2020));
+
+    switch (OPTION(kDispColorCorrectionProfile)) {
+        case config::ColorCorrectionProfile::kSRGB:
+            wxDynamicCast(color_profile_srgb, wxRadioButton)->SetValue(true);
+            break;
+        case config::ColorCorrectionProfile::kDCI:
+            wxDynamicCast(color_profile_dci, wxRadioButton)->SetValue(true);
+            break;
+        case config::ColorCorrectionProfile::kRec2020:
+            wxDynamicCast(color_profile_rec2020, wxRadioButton)->SetValue(true);
+            break;
+        default:
+            wxLogError(_("Invalid color correction profile"));
+    }
+
+    wxSlider* gba_darken_slider = GetValidatedChild<wxSlider>("GBADarken");
+    gba_darken_slider->SetValidator(widgets::OptionUnsignedValidator(config::OptionID::kGBADarken));
+    gba_darken_slider->SetToolTip(wxString::Format("%d", gba_darken_slider->GetValue()));
+    gba_darken_slider->Bind(wxEVT_SLIDER, std::bind(UpdateSliderTooltip, gba_darken_slider, std::placeholders::_1));
+    gba_darken_slider->Bind(wxEVT_ENTER_WINDOW, std::bind(UpdateSliderTooltipOnHover, gba_darken_slider, std::placeholders::_1));
+
+    wxSlider* gbc_lighten_slider = GetValidatedChild<wxSlider>("GBCLighten");
+    gbc_lighten_slider->SetValidator(widgets::OptionUnsignedValidator(config::OptionID::kGBLighten));
+    gbc_lighten_slider->SetToolTip(wxString::Format("%d", gbc_lighten_slider->GetValue()));
+    gbc_lighten_slider->Bind(wxEVT_SLIDER, std::bind(UpdateSliderTooltip, gbc_lighten_slider, std::placeholders::_1));
+    gbc_lighten_slider->Bind(wxEVT_ENTER_WINDOW, std::bind(UpdateSliderTooltipOnHover, gbc_lighten_slider, std::placeholders::_1));
+
     filter_selector_ = GetValidatedChild<wxChoice>("Filter");
     filter_selector_->SetValidator(FilterValidator());
     filter_selector_->Bind(wxEVT_CHOICE, &DisplayConfig::UpdatePlugin, this,
@@ -374,6 +504,14 @@ void DisplayConfig::OnDialogShowEvent(wxShowEvent& event) {
 
     if (event.IsShown()) {
         PopulatePluginOptions();
+
+        // Set initial SDL options visibility based on current selection
+        wxRadioButton* sdl_button = wxDynamicCast(GetValidatedChild("OutputSDL"), wxRadioButton);
+        if (sdl_button && sdl_button->GetValue()) {
+            ShowSDLOptions();
+        } else {
+            HideSDLOptions();
+        }
     } else {
         StopPluginHandler();
     }
@@ -439,8 +577,6 @@ void DisplayConfig::PopulatePluginOptions() {
 
     plugin_selector_->SetValidator(PluginSelectorValidator());
     ShowPluginOptions();
-
-    Fit();
 }
 
 void DisplayConfig::StopPluginHandler() {
@@ -549,4 +685,33 @@ void DisplayConfig::ShowPluginOptions() {
     }
 }
 
+void DisplayConfig::HideSDLOptions() {
+    sdlrenderer_label_->Hide();
+    sdlrenderer_selector_->Hide();
+    sdlpixelart_checkbox_->Hide();
+}
+
+void DisplayConfig::ShowSDLOptions() {
+    sdlrenderer_label_->Show();
+    sdlrenderer_selector_->Show();
+#if defined(ENABLE_SDL3) && defined(HAVE_SDL3_PIXELART)
+    sdlpixelart_checkbox_->Show();
+#endif
+}
+
+void DisplayConfig::UpdateSDLOptionsVisibility(wxCommandEvent& event) {
+    // Check if SDL output module is selected
+    wxRadioButton* sdl_button = wxDynamicCast(GetValidatedChild("OutputSDL"), wxRadioButton);
+    if (sdl_button && sdl_button->GetValue()) {
+        ShowSDLOptions();
+    } else {
+        HideSDLOptions();
+    }
+
+    Layout();
+
+    // Let the event propagate
+    event.Skip();
+}
+
 }  // namespace dialogs
