@@ -54,6 +54,7 @@ void SaveOption(config::Option* option) {
         case config::Option::Type::kFilter:
         case config::Option::Type::kInterframe:
         case config::Option::Type::kRenderMethod:
+        case config::Option::Type::kColorCorrectionProfile:
         case config::Option::Type::kAudioApi:
         case config::Option::Type::kAudioRate:
             cfg->Write(option->config_name(), option->GetEnumString());
@@ -290,6 +291,7 @@ void load_opts(bool first_time_launch) {
         case config::Option::Type::kFilter:
         case config::Option::Type::kInterframe:
         case config::Option::Type::kRenderMethod:
+        case config::Option::Type::kColorCorrectionProfile:
         case config::Option::Type::kAudioApi:
         case config::Option::Type::kAudioRate: {
             wxString temp;
@@ -380,8 +382,11 @@ void load_opts(bool first_time_launch) {
                     OPTION(kGBALinkTimeout) = 500;
                 }
 #endif
-                // Previous default was true.
-                OPTION(kGBALCDFilter) = false;
+            }
+            case 1: { // up to 2.2.2 included.
+                // Previous defaults were false.
+                OPTION(kGBALCDFilter) = true;
+                OPTION(kGBLCDFilter)  = true;
             }
         }
         ini_version++;
@@ -475,6 +480,7 @@ void opt_set(const wxString& name, const wxString& val) {
         case config::Option::Type::kFilter:
         case config::Option::Type::kInterframe:
         case config::Option::Type::kRenderMethod:
+        case config::Option::Type::kColorCorrectionProfile:
         case config::Option::Type::kAudioApi:
         case config::Option::Type::kAudioRate:
             opt->SetEnumString(val);
