@@ -150,7 +150,7 @@ private:
     config::EmulatedGamepad emulated_gamepad_;
 
     wxPathList config_path;
-    char* home = nullptr;
+    char* home = NULL;
 
     widgets::SdlPoller sdl_poller_;
     widgets::KeyboardInputHandler keyboard_input_handler_;
@@ -360,6 +360,8 @@ private:
     void OnMoveStart(wxMoveEvent& event);
     void OnMoveEnd(wxMoveEvent& event);
     void OnSize(wxSizeEvent& event);
+    void OnIconize(wxIconizeEvent& event);
+
     // Load a named wxDialog from the XRC file
     wxDialog* LoadXRCDialog(const char* name);
 
@@ -569,7 +571,7 @@ protected:
     uint32_t mouse_active_time;
     wxPoint mouse_last_pos;
 #ifdef __WXMSW__
-    HMENU current_hmenu = nullptr;
+    HMENU current_hmenu = NULL;
 #endif
 
     DECLARE_DYNAMIC_CLASS(GameArea)
@@ -611,7 +613,6 @@ public:
     virtual void OnSize(wxSizeEvent& ev);
     wxWindow* GetWindow() { return dynamic_cast<wxWindow*>(this); }
     virtual bool Destroy() { return GetWindow()->Destroy(); }
-    wxFrame *d3dframe = NULL;
 
 protected:
     virtual void DrawArea(wxWindowDC&) = 0;
